/*
	pcscan.h

	% Keyboard scan codes

	OWL-PC 1.2
	Copyright (c) 1986, 1987, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	12/20/88 ted:	Extracted from scancode.h so scancode can conditionally
					include it if the version is DOS.

	 6/07/89 jmd	Added new mouse codes

	 9/27/89 ted	Redefined KEY_, MOU_, HARD_, and BOB_ codes to not be ASCII.
	11/08/89 jmd	re-Redefined KEY_, MOU_, HARD_, and BOB_ codes to not be ASCII.
					(use 0x01xx instead of 0xFExx because many times scancodes
					 are defined as ints,  skip 0x011B; it's ESC)
	 1/24/90 ted	Added HELP_KEY dummy scancode.
	 3/16/90 jmd	Added KEY_INVALID scancode.
	 5/12/90 jmd	converted to new scheme, moved pseudocodes to scancode.h
					changed values for ctrl alphabet
	 9/06/90 jmd	returned to direct scancode use
	 9/09/90 ted	added FN10/FN11 scancodes
	 9/10/90 ted	added CTRL_UP,DN,INS,DEL codes, KEY_DUMMY macro.
	10/04/90 jmd	added casts to int to avoid turbo C++ complaints
	10/16/90 jmd	added KEY_DUMMYVAL
*/
/* -------------------------------------------------------------------------- */

/* Special codes for users and pseudo scancodes */
/* Use key numbers for ScrollLock, NumLock, CapsLock; they're not used otherwise */
#define KEY_USERVAL			0x4500
#define KEY_PSEUDOVAL		0x4600
#define KEY_DUMMYVAL		0x3A00

#define KEY_DUMMY(n)		(KEY_DUMMYVAL + (n))

/* Special HELP key not found on DOS machines' keyboards */
#define HELP_KEY		KEY_DUMMY(0)

/* Pseudo - ascii keys */
#define TAB				((int) 0x0F09)
#define BACKSPACE		((int) 0x0E08)
#define ENTER			((int) 0x1C0D)
#define ESC				((int) 0x011B)
#define GREYPLUS		((int) 0x4E2B)
#define GREYMINUS		((int) 0x4A2D)

/* Other special PC - defined keys */
#define CTRL_BACKSPACE	((int) 0x0E7F)
#define	PRTSC			((int) 0x372A)

#define	CTRL_PRTSC		((int) 0x7200)
#define SHFT_TAB		((int) 0x0F00)
#define	CTRL_2			((int) 0x0300)

#define	CTRL_6			((int) 0x071E)
#define	CTRL_HYPHEN		((int) 0x0C1F)
#define CTRL_BS			((int) 0x2B1C)		/* back slash */

/* 	Cursor Keys		*/

#define LEFT			((int) 0x4B00)
#define RIGHT			((int) 0x4D00)
#define UP				((int) 0x4800)
#define DOWN			((int) 0x5000)
#define HOME			((int) 0x4700)
#define END				((int) 0x4F00)
#define PGUP			((int) 0x4900)
#define PGDN			((int) 0x5100)
#define INS				((int) 0x5200)
#define DEL				((int) 0x5300)

/* CTRL  Cursor keys */

#define CTRL_LEFT		((int) 0x7300)
#define CTRL_RIGHT		((int) 0x7400)
#define CTRL_UP			((int) 0x8D00)
#define CTRL_DOWN		((int) 0x9100)
#define CTRL_HOME		((int) 0x7700)
#define CTRL_END		((int) 0x7500)
#define CTRL_PGUP		((int) 0x8400)
#define CTRL_PGDN		((int) 0x7600)
#define CTRL_INS		((int) 0x9200)
#define CTRL_DEL		((int) 0x9300)

/* Shift Cursor keys */

#define SHFT_LEFT		((int) 0x4B34)
#define SHFT_RIGHT		((int) 0x4D36)
#define SHFT_UP			((int) 0x4838)
#define SHFT_DOWN		((int) 0x5032)
#define SHFT_HOME		((int) 0x4737)
#define SHFT_END		((int) 0x4F31)
#define SHFT_PGUP		((int) 0x4939)
#define SHFT_PGDN		((int) 0x5133)
#define SHFT_INS		((int) 0x5230)
#define SHFT_DEL		((int) 0x532E)

/* Function Keys	*/

#define FN1				((int) 0x3B00)
#define FN2				((int) 0x3C00)
#define FN3				((int) 0x3D00)
#define FN4				((int) 0x3E00)
#define FN5				((int) 0x3F00)
#define FN6				((int) 0x4000)
#define FN7				((int) 0x4100)
#define FN8				((int) 0x4200)
#define FN9				((int) 0x4300)
#define FN10			((int) 0x4400)
#define FN11			((int) 0x8500)
#define FN12			((int) 0x8600)

/* Shift Function Keys	*/

#define SHFT_FN1		((int) 0x5400)
#define SHFT_FN2		((int) 0x5500)
#define SHFT_FN3		((int) 0x5600)
#define SHFT_FN4		((int) 0x5700)
#define SHFT_FN5		((int) 0x5800)
#define SHFT_FN6		((int) 0x5900)
#define SHFT_FN7		((int) 0x5A00)
#define SHFT_FN8		((int) 0x5B00)
#define SHFT_FN9		((int) 0x5C00)
#define SHFT_FN10		((int) 0x5D00)
#define SHFT_FN11		((int) 0x8700)
#define SHFT_FN12		((int) 0x8800)

/* CTRL  Function Keys	*/

#define CTRL_FN1		((int) 0x5E00)
#define CTRL_FN2		((int) 0x5F00)
#define CTRL_FN3		((int) 0x6000)
#define CTRL_FN4		((int) 0x6100)
#define CTRL_FN5		((int) 0x6200)
#define CTRL_FN6		((int) 0x6300)
#define CTRL_FN7		((int) 0x6400)
#define CTRL_FN8		((int) 0x6500)
#define CTRL_FN9		((int) 0x6600)
#define CTRL_FN10		((int) 0x6700)
#define CTRL_FN11		((int) 0x8900)
#define CTRL_FN12		((int) 0x8A00)

/* ALT   Function Keys	*/

#define ALT_FN1			((int) 0x6800)
#define ALT_FN2			((int) 0x6900)
#define ALT_FN3			((int) 0x6A00)
#define ALT_FN4			((int) 0x6B00)
#define ALT_FN5			((int) 0x6C00)
#define ALT_FN6			((int) 0x6D00)
#define ALT_FN7			((int) 0x6E00)
#define ALT_FN8			((int) 0x6F00)
#define ALT_FN9			((int) 0x7000)
#define ALT_FN10		((int) 0x7100)
#define ALT_FN11		((int) 0x8B00)
#define ALT_FN12		((int) 0x8C00)

/* CTRL  Alphabet		*/

#define CTRLC			((int) CTRL_C)		/* 1.0 compatible */

#define CTRL_A			((int) 0x1E01)
#define CTRL_B			((int) 0x3002)
#define CTRL_C			((int) 0x2E03)
#define CTRL_D			((int) 0x2004)
#define CTRL_E			((int) 0x1205)
#define CTRL_F			((int) 0x2106)
#define CTRL_G			((int) 0x2207)
#define CTRL_H			((int) 0x2308)
#define CTRL_I			((int) 0x1709)
#define CTRL_J			((int) 0x240A)
#define CTRL_K			((int) 0x250B)
#define CTRL_L			((int) 0x260C)
#define CTRL_M			((int) 0x320D)
#define CTRL_N			((int) 0x310E)
#define CTRL_O			((int) 0x180F)
#define CTRL_P			((int) 0x1910)
#define CTRL_Q			((int) 0x1011)
#define CTRL_R			((int) 0x1312)
#define CTRL_S			((int) 0x1F13)
#define CTRL_T			((int) 0x1414)
#define CTRL_U			((int) 0x1615)
#define CTRL_V			((int) 0x2F16)
#define CTRL_W			((int) 0x1117)
#define CTRL_X			((int) 0x2D18)
#define CTRL_Y			((int) 0x1519)
#define CTRL_Z			((int) 0x2C1A)

/* ALT   Alphabet		*/

#define	ALT_1			((int) 0x7800)
#define	ALT_2			((int) 0x7900)
#define	ALT_3			((int) 0x7A00)
#define	ALT_4			((int) 0x7B00)
#define	ALT_5			((int) 0x7C00)
#define	ALT_6			((int) 0x7D00)
#define	ALT_7			((int) 0x7E00)
#define	ALT_8			((int) 0x7F00)
#define	ALT_9			((int) 0x8000)
#define	ALT_0			((int) 0x8100)
#define	ALT_HYPHEN		((int) 0x8200)
#define	ALT_EQUALS		((int) 0x8300)

#define ALT_A			((int) 0x1E00)
#define ALT_B			((int) 0x3000)
#define ALT_C			((int) 0x2E00)
#define ALT_D			((int) 0x2000)
#define ALT_E			((int) 0x1200)
#define ALT_F			((int) 0x2100)
#define ALT_G			((int) 0x2200)
#define ALT_H			((int) 0x2300)
#define ALT_I			((int) 0x1700)
#define ALT_J			((int) 0x2400)
#define ALT_K			((int) 0x2500)
#define ALT_L			((int) 0x2600)
#define ALT_M			((int) 0x3200)
#define ALT_N			((int) 0x3100)
#define ALT_O			((int) 0x1800)
#define ALT_P			((int) 0x1900)
#define ALT_Q			((int) 0x1000)
#define ALT_R			((int) 0x1300)
#define ALT_S			((int) 0x1F00)
#define ALT_T			((int) 0x1400)
#define ALT_U			((int) 0x1600)
#define ALT_V			((int) 0x2F00)
#define ALT_W			((int) 0x1100)
#define ALT_X			((int) 0x2D00)
#define ALT_Y			((int) 0x1500)
#define ALT_Z			((int) 0x2C00)

