/*
	bdtitle.c	2/16/87

	% border with a title and scroll lights.

	OWL 1.2
	Copyright (c) 1986, 1987, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	11/22/87 jmd 	Replaced line characters with octal escape sequences
	 7/27/88 jmd 	Upgraded to new border stuff
	 9/12/88 jmd	Added in and out data
	11/13/88 jmd	Upgraded to new border stuff
	11/20/88 jmd	Added ID to obj struct
	12/20/88 jmd	removed SETSIZE msg

	 5/28/89 jmd	added Shadow support

	11/06/89 jmd	removed DoRaw macros
	 2/21/90 ted	Added SetCharSize(TRUE) because border won't work otherwise.
	 3/28/90 jmd	ansi-fied
*/

#include "oakhead.h"
#include "disppriv.h"
#include "strdecl.h"		/* for strwrap() */
#include "bordobj.h"
#include "bordod.h"


#define OUTER_BOX	"\332\304\267\272\274\315\324\263"
#define HZ_LINE		"\303\304\266"
#define VT_LINE		"\263\263\263"

/* scroll light macros */
#ifdef BORDER_CHARS
#	define	uplight(flag)	((flag) ? "\x1e" : " ")
#	define	downlight(flag)	((flag) ? "\x1f" : " ")
#else
#	define	uplight(flag)	((flag) ? "*" : " ")
#	define	downlight(flag)	((flag) ? "*" : " ")
#endif

/* border object data */

typedef struct _bdtitleod {
	border_od	bdd;		  	/* common object super class */
	boolean  	up;				/* up and down lights */
	boolean		down;
	int			theight;		/* title height */

} bdtitle_od;

int bd_title(VOID *objdata, int msg, VOID *indata, VOID *outdata)
/*
	This is the "title" border used by some C-scape library
	routines.
	It is a border with double lines on the right and lower
	sides and single lines on the left and top sides.
	There is a title and two "lights" (defined in bdf_struct).
	The title is a string of varying length with '\n's at the 
	end of each line.
	The up light and down light are booleans.  They are 
	designed to display up and down arrows to indicate whether or
	not there is more text to display in the enclosed sed.
	A TRUE value indicates that the light should be visible.
	Example:
					+---------------+
					| Title         |
					|---------------|
	Up light ----->	|*|             |
					| |             |
	Down light --->	|*|             |
					+---------------+
*/
{
	bdtitle_od  *bdtitled;
	ocbox		 cbox;
	ptd_struct 	*ptd;
	int			 tw;
	boolean		 oldup, olddown;
	boolean		 left, right;		/* dummy values */
	byte		 attr;

	bdtitled = (bdtitle_od *)objdata;

	switch(msg) {
	case OBJM_GETDATASIZE:
		((ogds_struct *) outdata)->odsize = sizeof(bdtitle_od);
		((ogds_struct *) outdata)->xdsize = sizeof(border_xd);
		((ogds_struct *) outdata)->id = ID_BDTITLE;
		break;

	case OBJM_OPEN:
		if (!border_Class(&bdtitled->bdd, msg, indata, outdata)) {
			return(FALSE);
		}

		bdtitled->theight = 0;
		win_SetCharSize(bdtitled->bdd.win, TRUE);	/* This bord must be charsize */
		bord_SetSides(bdtitled->bdd.win, - (bdtitled->theight + 1), -3, 1, 1);
		break;

	case BDM_RESIZE:
		/* our window has changed size, reset the scroll lights */
		bord_GetLights(bdtitled->bdd.win, &bdtitled->up, &bdtitled->down, &left, &right);
		break;

	case BDM_SETTITLE:
		if (bdtitled->bdd.title != NULL) {
			ofree(OA_BDTITLE, (VOID *) bdtitled->bdd.title);
			bdtitled->bdd.title = NULL;
		}

		if (indata == NULL) {
			bord_SetSides(bdtitled->bdd.win, - (bdtitled->theight + 1), -3, 1, 1);
			break;
		}

		tw = bord_GetWidth(bdtitled->bdd.win) - 2;
		if ((bdtitled->bdd.title = strwrap((char *) indata, &bdtitled->theight, tw)) == NULL) {
			return(FALSE);
		}

		/* add for line at bottom of title */
		(bdtitled->theight)++;

		/* adjust the border size */
		bord_SetSides(bdtitled->bdd.win, - (bdtitled->theight + 1), -3, 1, 1);
		break;

	case BDM_SCROLL:
		/* adjust the up and down lights */
		oldup = bdtitled->up;
		olddown = bdtitled->down;
		bord_GetLights(bdtitled->bdd.win, &bdtitled->up, &bdtitled->down, &left, &right);

		/* Only paint if arrows have changed */
		if (oldup != bdtitled->up || olddown != bdtitled->down) {
			/* paint up arrow */
			cbox.toprow   = 0;
			cbox.leftcol  = -2;
			cbox.botrow   = 0;
			cbox.rightcol = -2;
			win_PaintBox(bdtitled->bdd.win, &cbox);
			
			/* paint down arrow */
			cbox.toprow   = win_GetHeight(bdtitled->bdd.win) - 1;
			cbox.botrow   = cbox.toprow;
			win_PaintBox(bdtitled->bdd.win, &cbox);
		}
		break;

	case BDM_SHADOW:
	case BDM_DRAW:
		/* draw the border */
		ptd = (ptd_struct *)indata;

		attr = (msg == BDM_DRAW) ? 
			bord_GetAttr(bdtitled->bdd.win) : win_GetShadowAttr(bdtitled->bdd.win);

		/* draw the outer box */
		bord_GetBox(bdtitled->bdd.win, &cbox);
		ptd_DrawCharBox(ptd, OUTER_BOX, &cbox, attr);

		/* draw the vertical bars */
		cbox.toprow   = 0;
		cbox.leftcol  = -1;
		cbox.botrow   = win_GetHeight(bdtitled->bdd.win) - 1;
		cbox.rightcol = -1;
		ptd_DrawCharLine(ptd, VT_LINE, &cbox, attr);

		cbox.leftcol  = -2;		/* top and bottom don't change */
		cbox.rightcol = -2;
		ptd_DrawCharLine(ptd, "   ", &cbox, attr);
		
		/* draw the scroll lights */
		/* uplight */
		ptd_DrawString(ptd, 0, -2, uplight(bdtitled->up), attr, 1);
		/* downlight */
		ptd_DrawString(ptd, cbox.botrow, -2, downlight(bdtitled->down), attr, 1);

		/* draw the title if there is one */
		if (bdtitled->bdd.title != NULL) {
			/* draw the horizontal bar */
			cbox.toprow   = -1;
			cbox.leftcol  = -3;
			cbox.botrow   = cbox.toprow;
			cbox.rightcol = win_GetWidth(bdtitled->bdd.win);
			ptd_DrawCharLine(ptd, HZ_LINE, &cbox, attr);

			tw = win_GetWidth(bdtitled->bdd.win) + 2;
			bord_DrawTitle(ptd, -bdtitled->theight, -2, bdtitled->bdd.title, attr, tw);
		}
		/* else no break; pass message up to superclass */

	default:
		return(border_Class(&bdtitled->bdd, msg, indata, outdata));
	}
	return(1);
}

