/*
	bordout.c	5/20/89

	% Border Outline draw routines
	By JMD

	OWL 1.2
	Copyright (c) 1989, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 5/23/89 jmd	added "border features"
	 7/06/89 ted	Got rid of compiler warnings in EraseOutline.
	 7/06/89 ted	Fixed char-grid rounding.
	 8/26/89 jmd	Made bord_out routines public

	12/03/89 ted	Diddled with outline_round to make it use pix coords.
	12/04/89 jmd	added replaceable outline function
	 3/13/90 ted	Added call to ptd_DrawPixCharBox to fix pixel coords bug.
	 3/28/90 jmd	ansi-fied
	 2/01/90 pmcm	added _SavePixBox to draw function for using over grwins
	 8/30/90 ted	changed (curr_dmgr)->wmgr. references to wmgr_ macro calls.
	10/03/90 pmcm	added cache and flush to bord_EraseOutline and _DrawOutline
*/

#include "oakhead.h"
#include "disppriv.h"
#include "bordobj.h"

OGLOBAL bordout_fptr bord_drawoutfunc = FNULL;

OSTATIC void OWLPRIV outline_round(opbox *rectboxp, win_type win, opbox *rboxp);
/* -------------------------------------------------------------------------- */

#define OUTER_BOX	"\332\304\277\263\331\304\300\263"

void bord_DrawOutline(win_type win, opbox *rectboxp)
/*
	Draws a box directly to the display
	Used for moving windows.
	Calls bord_drawoutfunc to draw the border outline.
	If no draw func draws a character box.
*/
{
	ptd_struct	ptds;
	opbox	    relbox, pbox;

	/* Create our own ptd, as big as the display */
	ptds.win = disp_GetDispWin();
	ptds.emsgdata = NULL;
	ptds.relboxp = &relbox;

	win_GetPixBox(ptds.win, ptds.relboxp);

	outline_round(rectboxp, win, &pbox);

	disp_HideMouse();
	disp_Cache();

	/* Save the layer we are about to obscure */
	wmgr_SavePixBox(wmgr_botsyswin(), &pbox);

	/* Draw the outline */
	if (bord_drawoutfunc == FNULL || !(*bord_drawoutfunc)(&ptds, win, &pbox)) {
		/* if no draw func, or if it fails, draw a char box */
		ptd_DrawPixCharBox(&ptds, &pbox, OUTER_BOX, win_GetFont(win), win_GetAttr(win));
	}
	disp_Flush();
	disp_ShowMouse();
}
/* -------------------------------------------------------------------------- */

void bord_EraseOutline(win_type win, opbox *rectboxp)
/*
	Erases a border outline.
	Used for moving windows.
*/
{
 	opbox	pbox, dbox;
	opcoord	fw, fh;

	/* round rectbox to character grid by converting from pix to char coords */
	/*  and back again (into pbox). */
	outline_round(rectboxp, win, &pbox);

	/* repaint afflicted area */

	fw = win_GetFontWidth(win);
	fh = win_GetFontHeight(win);

	disp_Cache();

 	/* restore four strips */
	wmgr_PaintPixBox(wmgr_botsyswin(),
		opbox_set(&dbox, pbox.xmin, pbox.ymin, pbox.xmax, pbox.ymin + fh));

	wmgr_PaintPixBox(wmgr_botsyswin(),
		opbox_set(&dbox, pbox.xmin, pbox.ymax - fh, pbox.xmax, pbox.ymax));

	pbox.ymin += fh;
	pbox.ymax -= fh;

	wmgr_PaintPixBox(wmgr_botsyswin(),
		opbox_set(&dbox, pbox.xmin, pbox.ymin, pbox.xmin + fw, pbox.ymax));

	wmgr_PaintPixBox(wmgr_botsyswin(),
		opbox_set(&dbox, pbox.xmax - fw, pbox.ymin, pbox.xmax, pbox.ymax));

	disp_Flush();
}
/* -------------------------------------------------------------------------- */

static void OWLPRIV outline_round(opbox *rectboxp, win_type win, opbox *rboxp)
/*
	Round the coordinates for a character-width border to the right positions.
*/
{
	rboxp->xmin = rectboxp->xmin;
	rboxp->ymin = rectboxp->ymin;
	rboxp->xmax = opbox_GetWidth(rectboxp);
	rboxp->ymax = opbox_GetHeight(rectboxp);

	if (!disp_TextFree()) {
		opcoord_GridRound(&rboxp->xmin, &rboxp->ymin, win_GetFont(win));
	}
	if (!disp_TextFree() || win_IsCharSize(win)) {
		rboxp->xmax += win_GetFontWidth(win) - 1;	/* Round up instead of down */
		rboxp->ymax += win_GetFontHeight(win) - 1;
		opcoord_GridRound(&rboxp->xmax, &rboxp->ymax, win_GetFont(win));
	}
	rboxp->xmax += rboxp->xmin;
	rboxp->ymax += rboxp->ymin;
}
/* -------------------------------------------------------------------------- */

