/*
	dispclr.c

	% Display clear.

	 3/24/89  by Ted.
	Extracted from wmgr.c as wmgr_Clear

	OWL 1.2
	Copyright (c) 1988, 1989 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 3/28/90 jmd	ansi-fied
	 6/22/90 ted	added "void" to no-parameter function per ansii.
	 9/26/90 pmcm	added cls() in disp_Repaint for non-DOS systems
	10/22/90 ted	added O_FORCE_REPAINT flag to fix apollo repaint bug.
*/

#include "oakhead.h"
#include "disppriv.h"

/* -------------------------------------------------------------------------- */

void disp_Clear(opixval color)
/*
	Clear the whole display flat out, regardless of what windows are present,
	and sending no messages to any windows.
*/
{
	opbox box;
	ptd_struct ptds;

	owl_Assert(disp_Ok(), OE_DC_DISP);

	ptds.win = disp_GetDispWin();
	ptds.emsgdata = NULL;
	ptds.relboxp = &box;
	box.ymin = 0;
	box.xmin = 0;
	box.ymax = disp_GetPixHeight();
	box.xmax = disp_GetPixWidth();

	disp_Cache();
	ptd_Clear(&ptds, color);
	disp_Flush();
}
/* -------------------------------------------------------------------------- */

void disp_Repaint(void)
{
	owl_Assert(disp_Ok(), OE_DR_DISP);

#ifdef O_FORCE_REPAINT
	/* 	clear the display on non-DOS systems.  
		E.g., UNIX curses and VAX/VMS SMG require this in order to 
		refresh the	display in the case that the display have changed 
		as a result	of output that did not go thru curses or SMG--i.e., 
		the display has been alatered but their internal representations 
		show no change.
	*/
	cls();
#endif

	wmgr_PaintPixBox(wmgr_botsyswin(), win_pixboxp(disp_GetDispWin()));
}
/* -------------------------------------------------------------------------- */

