/*
  	kbidle.c	10/02/87

	% kb_Idle
	Replaces normal kb_Read() function with new handler.

	OWL 1.2
	Copyright (c) 1987, 1988, 1989 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 9/09/88 jmd	Ported to oakland lib
	 9/19/88 jmd	Added messages to idle func
	10/04/88 jmd	Added support for kb_Check
	10/27/88 jmd	Fixed bug in ReadHandler
	12/01/88 jmd	modified for new KeyReadEvent
	 2/07/89 Ted	added timeout wait arg to hCheckEvent function.
	 7/07/89 gam	changed NULL to FNULL where applicable

	11/30/89 jmd	fixed CheckEvent test (now uses KEY_NONE)
	 3/28/90 jmd	ansi-fied
	 4/14/90 jmd	added return value to idle_func
	 4/24/90 ted	added wait processing to idle_func
	 5/12/90 jmd 	changed scancodes to ints
*/

#include "oakhead.h"
#include "disppriv.h"
#include "digutil.h"	/* for dig_SubHsecs */
#include "kbidle.h"
#include "scancode.h"	/* for mouse pseudo-scancodes */

OSTATIC int kb_ReadHandler(moupos_struct *mposp);
OSTATIC int kb_CheckHandler(unsigned wait);

/* static function pointers */
static idle_fptr				kIdle_fptr		 = FNULL;
static dig_hReadEvent_func		((*hReadEventFptr)) = FNULL;
static dig_hCheckEvent_func		((*hCheckEventFptr)) = FNULL;

static kistash = KEY_NONE;

/* -------------------------------------------------------------------------- */

void kb_Idle(idle_fptr idle)
/*
	Sets up idle key function.

	The idle function is called with an IDLE_START message.

	If idle == NULL turn off the idle function and call it with
	an IDLE_STOP message.
*/
{
	if (hReadEventFptr != FNULL) {
		/* reset kb functions */
		(*kIdle_fptr)(IDLE_STOP, 0);
		curr_dmgr->disp.dig.hReadEvent = hReadEventFptr;
		curr_dmgr->disp.dig.hCheckEvent = hCheckEventFptr;
		hReadEventFptr = FNULL;
		hCheckEventFptr = FNULL;
		kIdle_fptr = FNULL;
	}

	if (idle != FNULL) {
		/* save old keyboard handlers */
		hReadEventFptr = curr_dmgr->disp.dig.hReadEvent;
		hCheckEventFptr = curr_dmgr->disp.dig.hCheckEvent;

		/* set new keyboard handlers */
		curr_dmgr->disp.dig.hReadEvent = kb_ReadHandler;
		curr_dmgr->disp.dig.hCheckEvent = kb_CheckHandler;
		kIdle_fptr = idle;
		(*kIdle_fptr)(IDLE_START, 0);
	}
}
/* -------------------------------------------------------------------------- */

static int kb_CheckHandler(unsigned wait)
/*
	Calls user idle function then calls original kb_Check routine
	idle function is called with an IDLE_RUN message.
*/
{
	unsigned itime, elaps;

	if (wait != 0 && wait != (unsigned) -1) {
		itime = hard_Timer();
	}
	if ((kistash =(*kIdle_fptr)(IDLE_CHECK, wait)) != KEY_NONE) {
		return(kistash);
	}
	if (wait != 0 && wait != (unsigned) -1) {
		elaps = dig_SubHsecs(itime, hard_Timer());
		if (elaps > wait) {
			wait = 0;
		}
		else {
			wait -= elaps;
		}
	}
	return((*hCheckEventFptr)(wait));
}
/* -------------------------------------------------------------------------- */

static int kb_ReadHandler(moupos_struct *mposp)
/*
	Calls user idle function while real kb_Check routine returns FALSE.
	Calls original kb_Read when a keystroke is available
	idle function is called with an IDLE_RUN message.

	The idle function should return KEY_NONE.  Any other value is
	returned via kb_Read()
*/
{
	int key, tstash;

	if (kistash != KEY_NONE) {
		tstash = kistash;
		kistash = KEY_NONE;
		return(tstash);
	}
	while ((*hCheckEventFptr)(0) == KEY_NONE) {
		if ((key = (*kIdle_fptr)(IDLE_READ, 0)) != KEY_NONE) {
			return(key);
		}
	}
	return((*hReadEventFptr)(mposp));
}
/* -------------------------------------------------------------------------- */

