/*
  	kblist.c

	% 	kb_keylist		used by kb_Record() and kb_Macro()

	OWL 1.2
	Copyright (c) 1990 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 9/21/90 pmcm	made kb_keylist OGLOBAL
	10/04/90 pmcm	added FN11, FN12, CTRL_DOWN, CTRL_UP, CTRL_INS, 
					CTRL_DEL, HELP_KEY, and VAX/VMS-SMG specific codes
*/

#include "oakhead.h"
#include "disppriv.h"

#include "scancode.h"
#include "kbrecord.h"

/*
	scancode, name string, macro size, macro

	the last entry in each structure is internal and is not set by the 
	programmer; each should be : NULL.

	names MUST not be longer than KB_NAME_MAXLEN (22).

	the list MUST end with KB_LIST_END.

	scancode should NEVER == 0 (used in KB_LIST_END), but can be set to
	KEY_NONE for macros that not available through the keyboard.

	if the last entry in each struct, macro size == -1 then no macro is 
	allowed	for that key.
	use this if you wish to suppress macros for only a few keys.

	specific key macros can also be suppressed by removing (or commenting out)
	that key's entry in the list.
	this would also speed keystroke retrieval.

	the last 2 entries in the list KEY_PAUSE and KEY_PROMPT are for 
	kb_Record() and don't apply to macros.
*/

OGLOBAL key_struct kb_keylist[] = {

	{ HELP_KEY,			"HELP_KEY", 	 	0, NULL },
	{ HOME,				"HOME", 			0, NULL },
	{ END,				"END", 				0, NULL },
	{ LEFT,				"LEFT", 			0, NULL },
	{ RIGHT,			"RIGHT", 			0, NULL },
	{ UP,				"UP", 				0, NULL },
	{ DOWN,				"DOWN", 			0, NULL },
	{ PGUP,				"PGUP", 			0, NULL },
	{ PGDN,				"PGDN", 			0, NULL },
	{ INS,				"INS", 				0, NULL },
	{ DEL,				"DEL", 				0, NULL },
	{ TAB,				"TAB", 				0, NULL },
	{ BACKSPACE,		"BACKSPACE", 		0, NULL },
	{ ENTER,			"ENTER", 			0, NULL },
	{ ESC,				"ESC", 				0, NULL },
	{ PRTSC,			"PRTSC", 			0, NULL },
	{ GREYPLUS,			"GREYPLUS", 		0, NULL },
	{ GREYMINUS,		"GREYMINUS", 		0, NULL },
	{ CTRL_HOME,		"CTRL_HOME", 		0, NULL },
	{ CTRL_END,			"CTRL_END", 		0, NULL },
	{ CTRL_LEFT,		"CTRL_LEFT", 		0, NULL },
	{ CTRL_RIGHT,		"CTRL_RIGHT", 		0, NULL },
	{ CTRL_UP,			"CTRL_UP", 			0, NULL },
	{ CTRL_DOWN,		"CTRL_DOWN", 		0, NULL },
	{ CTRL_PGUP,		"CTRL_PGUP", 		0, NULL },
	{ CTRL_PGDN,		"CTRL_PGDN", 		0, NULL },
	{ CTRL_INS,			"CTRL_INS", 		0, NULL },
	{ CTRL_DEL,			"CTRL_DEL", 		0, NULL },
	{ CTRL_BACKSPACE,	"CTRL_BACKSPACE", 	0, NULL },
	{ CTRL_PRTSC,		"CTRL_PRTSC", 		0, NULL },
	{ SHFT_HOME,		"SHFT_HOME", 		0, NULL },
	{ SHFT_END,			"SHFT_END", 		0, NULL },
	{ SHFT_LEFT,		"SHFT_LEFT", 		0, NULL },
	{ SHFT_RIGHT,		"SHFT_RIGHT", 		0, NULL },
	{ SHFT_UP,			"SHFT_UP", 			0, NULL },
	{ SHFT_DOWN,		"SHFT_DOWN", 		0, NULL },
	{ SHFT_PGUP,		"SHFT_PGUP", 		0, NULL },
	{ SHFT_PGDN,		"SHFT_PGDN", 		0, NULL },
	{ SHFT_INS,			"SHFT_INS", 		0, NULL },
	{ SHFT_DEL,			"SHFT_DEL", 		0, NULL },
	{ SHFT_TAB,			"SHFT_TAB", 		0, NULL },
	{ FN1,				"FN1", 				0, NULL },
	{ FN2,				"FN2", 				0, NULL },
	{ FN3,				"FN3", 				0, NULL },
	{ FN4,				"FN4", 				0, NULL },
	{ FN5,				"FN5", 				0, NULL },
	{ FN6,				"FN6", 				0, NULL },
	{ FN7,				"FN7", 				0, NULL },
	{ FN8,				"FN8", 				0, NULL },
	{ FN9,				"FN9", 				0, NULL },
	{ FN10,				"FN10", 			0, NULL },
	{ FN11,				"FN11", 			0, NULL },
	{ FN12,				"FN12", 			0, NULL },
	{ SHFT_FN1,			"SHFT_FN1", 		0, NULL },
	{ SHFT_FN2,			"SHFT_FN2", 		0, NULL },
	{ SHFT_FN3,			"SHFT_FN3", 		0, NULL },
	{ SHFT_FN4,			"SHFT_FN4", 		0, NULL },
	{ SHFT_FN5,			"SHFT_FN5", 		0, NULL },
	{ SHFT_FN6,			"SHFT_FN6", 		0, NULL },
	{ SHFT_FN7,			"SHFT_FN7", 		0, NULL },
	{ SHFT_FN8,			"SHFT_FN8", 		0, NULL },
	{ SHFT_FN9,			"SHFT_FN9", 		0, NULL },
	{ SHFT_FN10,  		"SHFT_FN10", 		0, NULL },
	{ SHFT_FN11,		"SHFT_FN11",  		0, NULL },
	{ SHFT_FN12,		"SHFT_FN12",  		0, NULL },
	{ CTRL_FN1,			"CTRL_FN1", 		0, NULL },
	{ CTRL_FN2,			"CTRL_FN2", 		0, NULL },
	{ CTRL_FN3,			"CTRL_FN3", 		0, NULL },
	{ CTRL_FN5,			"CTRL_FN5", 		0, NULL },
	{ CTRL_FN6,			"CTRL_FN6", 		0, NULL },
	{ CTRL_FN7,			"CTRL_FN7", 		0, NULL },
	{ CTRL_FN8,			"CTRL_FN8", 		0, NULL },
	{ CTRL_FN9,			"CTRL_FN9", 		0, NULL },
	{ CTRL_FN10,  		"CTRL_FN10", 		0, NULL },
	{ CTRL_FN11,		"CTRL_FN11", 	 	0, NULL },
	{ CTRL_FN12,		"CTRL_FN12", 	 	0, NULL },
	{ ALT_FN1,			"ALT_FN1", 			0, NULL },
	{ ALT_FN2,			"ALT_FN2", 			0, NULL },
	{ ALT_FN3,			"ALT_FN3", 			0, NULL },
	{ ALT_FN4,			"ALT_FN4", 			0, NULL },
	{ ALT_FN5,			"ALT_FN5", 			0, NULL },
	{ ALT_FN6,			"ALT_FN6", 			0, NULL },
	{ ALT_FN7,			"ALT_FN7", 			0, NULL },
	{ ALT_FN8,			"ALT_FN8", 			0, NULL },
	{ ALT_FN9,			"ALT_FN9", 			0, NULL },
	{ ALT_FN10,			"ALT_FN10", 		0, NULL },
	{ ALT_FN11,			"ALT_FN11", 		0, NULL },
	{ ALT_FN12,			"ALT_FN12", 		0, NULL },
	{ CTRL_2,			"CTRL_2", 			0, NULL },
	{ CTRL_6,			"CTRL_6", 			0, NULL },
	{ CTRL_HYPHEN,		"CTRL_HYPHEN", 		0, NULL },
	{ CTRL_BS,			"CTRL_BS", 			0, NULL },
	{ CTRL_A,			"CTRL_A", 			0, NULL },
	{ CTRL_B,			"CTRL_B", 			0, NULL },
	{ CTRL_C,			"CTRL_C", 			0, NULL },
	{ CTRL_D,			"CTRL_D", 			0, NULL },
	{ CTRL_E,			"CTRL_E", 			0, NULL },
	{ CTRL_F,			"CTRL_F", 			0, NULL },
	{ CTRL_G,			"CTRL_G", 			0, NULL },
	{ CTRL_H,			"CTRL_H", 			0, NULL },
	{ CTRL_I,			"CTRL_I", 			0, NULL },
	{ CTRL_J,			"CTRL_J", 			0, NULL },
	{ CTRL_K,			"CTRL_K", 			0, NULL },
	{ CTRL_L,			"CTRL_L", 			0, NULL },
	{ CTRL_M,			"CTRL_M", 			0, NULL },
	{ CTRL_N,			"CTRL_N", 			0, NULL },
	{ CTRL_O,			"CTRL_O", 			0, NULL },
	{ CTRL_P,			"CTRL_P", 			0, NULL },
	{ CTRL_Q,			"CTRL_Q", 			0, NULL },
	{ CTRL_R,			"CTRL_R", 			0, NULL },
	{ CTRL_S,			"CTRL_S", 			0, NULL },
	{ CTRL_T,			"CTRL_T", 			0, NULL },
	{ CTRL_U,			"CTRL_U", 			0, NULL },
	{ CTRL_V,			"CTRL_V", 			0, NULL },
	{ CTRL_W,			"CTRL_W", 			0, NULL },
	{ CTRL_X,			"CTRL_X", 			0, NULL },
	{ CTRL_Y,			"CTRL_Y", 			0, NULL },
	{ CTRL_Z,			"CTRL_Z", 			0, NULL },
	{ ALT_1,			"ALT_1", 			0, NULL },
	{ ALT_2,			"ALT_2", 			0, NULL },
	{ ALT_3,			"ALT_3", 			0, NULL },
	{ ALT_4,			"ALT_4", 			0, NULL },
	{ ALT_5,			"ALT_5", 			0, NULL },
	{ ALT_6,			"ALT_6", 			0, NULL },
	{ ALT_7,			"ALT_7", 			0, NULL },
	{ ALT_8,			"ALT_8", 			0, NULL },
	{ ALT_9,			"ALT_9", 			0, NULL },
	{ ALT_0,			"ALT_0", 			0, NULL },
	{ ALT_HYPHEN,		"ALT_HYPHEN", 		0, NULL },
	{ ALT_EQUALS,		"ALT_EQUALS", 		0, NULL },
	{ ALT_A,			"ALT_A", 			0, NULL },
	{ ALT_B,			"ALT_B", 			0, NULL },
	{ ALT_C,			"ALT_C", 			0, NULL },
	{ ALT_D,			"ALT_D", 			0, NULL },
	{ ALT_E,			"ALT_E", 			0, NULL },
	{ ALT_F,			"ALT_F", 			0, NULL },
	{ ALT_G,			"ALT_G", 			0, NULL },
	{ ALT_H,			"ALT_H", 			0, NULL },
	{ ALT_I,			"ALT_I", 			0, NULL },
	{ ALT_J,			"ALT_J", 			0, NULL },
	{ ALT_K,			"ALT_K", 			0, NULL },
	{ ALT_L,			"ALT_L", 			0, NULL },
	{ ALT_M,			"ALT_M", 			0, NULL },
	{ ALT_N,			"ALT_N", 			0, NULL },
	{ ALT_O,			"ALT_O", 			0, NULL },
	{ ALT_P,			"ALT_P", 			0, NULL },
	{ ALT_Q,			"ALT_Q", 			0, NULL },
	{ ALT_R,			"ALT_R", 			0, NULL },
	{ ALT_S,			"ALT_S", 			0, NULL },
	{ ALT_T,			"ALT_T", 			0, NULL },
	{ ALT_U,			"ALT_U", 			0, NULL },
	{ ALT_V,			"ALT_V", 			0, NULL },
	{ ALT_W,			"ALT_W", 			0, NULL },
	{ ALT_X,			"ALT_X", 			0, NULL },
	{ ALT_Y,			"ALT_Y", 			0, NULL },
	{ ALT_Z,			"ALT_Z", 			0, NULL },
	{ HARD_MEV,			"M", 				0, NULL },	/* mouse event    */

	{ KEY_PAUSE,		"PAUSE", 			0, NULL },	/* pause command  */
	{ KEY_PROMPT,		"PROMPT",			0, NULL },	/* prompt command */

#ifdef OAK_VMS
	{ E1,				"E1",				0, NULL },	/* HOME alias */
	{ E2,				"E2",				0, NULL },	/* INS 	alias */
	{ E3,				"E3",				0, NULL },	/* DEL 	alias */
	{ E4,				"E4",				0, NULL },	/* END 	alias */
	{ E5,				"E5",				0, NULL },	/* PGUP alias */
	{ E6,				"E6",				0, NULL },	/* PGDN alias */
	{ PF1,				"PF1",				0, NULL },
	{ PF2,				"PF2",				0, NULL },
	{ PF3,				"PF3",				0, NULL },
	{ PF4,				"PF4",				0, NULL },
	{ AMK_0,			"AMK_0",			0, NULL },
	{ AMK_1,			"AMK_1",			0, NULL },
	{ AMK_2,			"AMK_2",			0, NULL },
	{ AMK_3,			"AMK_3",			0, NULL },
	{ AMK_4,			"AMK_4",			0, NULL },
	{ AMK_5,			"AMK_5",			0, NULL },
	{ AMK_6,			"AMK_6",			0, NULL },
	{ AMK_7,			"AMK_7",			0, NULL },
	{ AMK_8,			"AMK_8",			0, NULL },
	{ AMK_9,			"AMK_9",			0, NULL },
	{ AMK_COMMA,		"AMK_COMMA",	 	0, NULL }, 	/* GREYPLUS 	alias	*/
	{ AMK_MINUS,		"AMK_MINUS",	 	0, NULL },	/* GREYMINUS 	alias	*/
	{ AMK_ENTER,		"AMK_ENTER",	 	0, NULL },	/* ENTER 		alias	*/
	{ AMK_PERIOD,		"AMK_PERIOD",	 	0, NULL },
	{ DO,				"DO",				0, NULL },	/* VAX/SMG key F16 */
#endif					 

	{ KB_LIST_END }				/* End of list, ignore keystroke  */
};
